"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Node = void 0;
class Node {
    path;
    gun;
    ref;
    constructor(path, gun) {
        this.path = path;
        this.gun = gun;
        const p = path.split('.');
        this.ref = p.reduce((previous, key) => previous.get(key), gun);
    }
    // TODO: Recursive execution
    encode(data) {
        if (!data || typeof data !== 'object')
            return data;
        return Object.keys(data).reduce((object, key) => {
            if (Array.isArray(data[key])) {
                object[`[]${key}`] = JSON.stringify(data[key]);
            }
            return object;
        }, {});
    }
    // TODO: Recursive execution
    decode(data) {
        if (!data || typeof data !== 'object')
            return data;
        return Object.keys(data).reduce((object, key) => {
            if (key.startsWith('[]')) {
                object[key.slice(2)] = JSON.parse(data[key]);
            }
            return object;
        }, {});
    }
    /**
     * Get parent node
     */
    get parent() {
        const path = this.path
            .split('.')
            .slice(0, -1)
            .join('.');
        const node = new Node(path, this.gun);
        if (node.path.length > 0)
            return node;
        return null;
    }
    /**
     * Set a value
     */
    set(value) {
        return new Promise((resolve, reject) => {
            this.ref.put({ $data: this.encode(value) }, ({ err, ok }) => {
                if (err !== undefined)
                    return reject(err);
                resolve(ok);
            });
        });
    }
    /**
     * Get next nodes
     * `await stack.node('a.b').set({hello: 'world'})`
     * `await stack.node('a').next() -> [Node({path: 'a.b'})]`
     */
    async next() {
        const keys = new Set();
        return new Promise((resolve) => {
            this.ref.map((value, key) => {
                if (!key.startsWith('$'))
                    keys.add(key);
                return {};
            }).once(() => resolve([...keys].map(key => new Node(`${this.path}.${key}`, this.gun))));
        });
    }
    /**
     * Listen for node data changes
     * @param listener handler fo new data
     */
    on(listener) {
        this.ref.on((value) => {
            if (value.$data !== undefined) {
                listener(this.decode(value.$data));
            }
        });
    }
    /**
     * Get data from node
     * @param timeout timeout for data fetching, will return null after timeout
     */
    get(timeout) {
        return new Promise((resolve) => {
            let timeoutInstance;
            if (timeout) {
                timeoutInstance = setTimeout(() => resolve(null), timeout);
            }
            this.ref.once((value) => {
                if (!value)
                    return resolve(null);
                if (value.$data) {
                    if (timeoutInstance) {
                        clearTimeout(timeoutInstance);
                    }
                    resolve(this.decode(value.$data));
                }
            }, timeout ? { wait: timeout } : undefined);
        });
    }
}
exports.Node = Node;
//# sourceMappingURL=node.js.map